'use strict';
const path = require('path');
const io_1 = require('./lib/io');
const attributes_1 = require('./lib/attributes');
const clean_1 = require('./lib/clean');
const fullTemplate = [
    '<?xml version="1.0" encoding="iso-8859-1"?>',
    '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">',
    '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink"$attrs>$icons</svg>'
].join('');
function makeSprite(dir, options) {
    if (!dir) {
        throw new Error('`dir` required');
    }
    let template = fullTemplate;
    options = Object.assign({
        ignore: [],
        parentAttrs: {},
        inline: false,
        iconAttrs: {},
        iconPrefix: '',
        iconSuffix: '',
        clean: {}
    }, options);
    options.clean = Object.assign({
        stripComment: false,
        stripDescription: false,
        stripEmptyDefinition: false,
        stripEmptyGroup: false,
        stripIndent: false,
        stripExtraAttributes: false,
        stripTitle: false,
        stripFill: false,
        stripStyles: false
    }, options.clean);
    options.ignore.push('!*.svg');
    if (options.inline) {
        options.parentAttrs = Object.assign({
            width: 0,
            height: 0,
            style: 'position:absolute'
        }, options.parentAttrs);
        template = `<svg$attrs>$icons</svg>`;
    }
    const parentAttrs = attributes_1.makeAttributes(options.parentAttrs);
    template = template.replace('$attrs', parentAttrs);
    return io_1.readdirPromise(dir, options.ignore)
        .then((files) => Promise.all(files.map((filepath) => {
        const name = path.basename(filepath);
        return io_1.readFilePromise(filepath, 'utf-8').then((content) => ({ name: name, content: content }));
    })))
        .then((files) => {
        let sprite = false;
        const icons = files.map((file) => {
            if (file.content.indexOf('<symbol') !== -1) {
                sprite = true;
                file.content = file.content.replace(/<\/*svg.*?>/g, '');
            }
            if (file.content.indexOf('<defs') !== -1) {
                sprite = true;
                file.content = file.content
                    .replace(/<\/*(?:svg|defs).*?>/g, '')
                    .replace(/<(\/*)g(.*)?>/g, '<$1svg$2>');
            }
            options.iconAttrs.id = options.iconPrefix + path.basename(file.name, '.svg') + options.iconSuffix;
            if (sprite) {
                delete options.iconAttrs.id;
            }
            file.content = clean_1.clean(file.content, options.clean);
            file.content = attributes_1.updateAttributes(file, options.iconAttrs);
            file.content = file.content.replace(/<(\/*)svg/g, '<$1symbol');
            sprite = false;
            return file.content;
        });
        template = template.replace('$icons', icons.join(''));
        return {
            sprite: template,
            write: (filepath) => io_1.writeFilePromise(filepath, template)
        };
    });
}
exports.makeSprite = makeSprite;
